@@ -1,4 +1,4 @@
-__version__ = "3.1.1"
+__version__ = "4.0.0"
 
 
 from .config_resolver import ConfigResolver  # noqa: F401
@@ -6,5 +6,9 @@ from .proc_wrapper import ProcWrapper  # noqa: F401
 from .proc_wrapper_params import CONFIG_MERGE_STRATEGY_SHALLOW  # noqa: F401
 from .proc_wrapper_params import ConfigResolverParams  # noqa: F401
 from .proc_wrapper_params import ProcWrapperParams, make_arg_parser  # noqa: F401
-from .runtime_metadata import RuntimeMetadata, RuntimeMetadataFetcher  # noqa: F401
+from .runtime_metadata import (  # noqa: F401
+    DefaultRuntimeMetadataFetcher,
+    RuntimeMetadata,
+    RuntimeMetadataFetcher,
+)
 from .status_updater import StatusUpdater  # noqa: F401
