@@ -3,7 +3,8 @@ import json
 import logging
 import os
 import re
-from typing import Any, Dict, List, NamedTuple, Optional, cast
+import shlex
+from typing import Any, Dict, List, Mapping, NamedTuple, Optional, Tuple, Union, cast
 
 from .common_utils import coalesce, encode_int, string_to_bool, string_to_int
 from .runtime_metadata import RuntimeMetadata
@@ -36,6 +37,10 @@ DEFAULT_RESOLVABLE_ENV_VAR_NAME_SUFFIX = "_FOR_PROC_WRAPPER_TO_RESOLVE"
 DEFAULT_RESOLVABLE_CONFIG_PROPERTY_NAME_PREFIX = ""
 DEFAULT_RESOLVABLE_CONFIG_PROPERTY_NAME_SUFFIX = "__to_resolve"
 
+SHELL_MODE_AUTO = "auto"
+SHELL_MODE_FORCE_ENABLE = "enable"
+SHELL_MODE_FORCE_DISABLE = "disable"
+DEFAULT_SHELL_MODE = SHELL_MODE_AUTO
 
 DEFAULT_STATUS_UPDATE_SOCKET_PORT = 2373
 DEFAULT_STATUS_UPDATE_MESSAGE_MAX_BYTES = 64 * 1024
@@ -50,6 +55,10 @@ DEFAULT_PROCESS_TERMINATION_GRACE_PERIOD_SECONDS = 30
 
 UNSET_INT_VALUE = -1000000
 
+SHELL_COMMAND_REGEX = re.compile(r"[^\w\-/. ]")
+
+# Detects sh, bash, csh, zsh, ash, dash, and fish shells
+SHELL_WRAPPER_REGEX = re.compile(r"^\s*(/\w+)*/(ba|c|z|fi|d?a)?sh\s+-c\s+")
 
 _logger = logging.getLogger(__name__)
 _logger.addHandler(logging.NullHandler())
@@ -123,7 +132,7 @@ class ConfigResolverParams:
             self.override_resolver_params_from_env()
 
     def override_resolver_params_from_env(
-        self, env: Optional[Dict[str, str]] = None
+        self, env: Optional[Mapping[str, str]] = None
     ) -> None:
         if env is None:
             env = dict(os.environ)
@@ -136,11 +145,11 @@ class ConfigResolverParams:
         )
 
         env_locations_in_env = env.get("PROC_WRAPPER_ENV_LOCATIONS")
-        if env_locations_in_env is not None:
+        if env_locations_in_env:
             self.env_locations = self.split_location_string(env_locations_in_env)
 
         config_locations_in_env = env.get("PROC_WRAPPER_CONFIG_LOCATIONS")
-        if config_locations_in_env is not None:
+        if config_locations_in_env:
             self.config_locations = self.split_location_string(config_locations_in_env)
 
         self.config_merge_strategy = env.get(
@@ -279,9 +288,21 @@ class ConfigResolverParams:
         # backslashes. Any occurrence of , or ; in a location string
         # must be backslash escaped. This doesn't handle the weird case
         # when a location contains "\," or "\;".
+        locations = locations.strip()
+
+        if not locations:
+            return []
+
         return [
-            location.replace(r"\,", ",").replace(r"\;", ";").replace(r"\\\\", r"\\")
-            for location in re.split(r"\s*(?<!(?<!\\)\\)[,;]\s*", locations)
+            x
+            for x in [
+                location.replace(r"\,", ",")
+                .replace(r"\;", ";")
+                .replace(r"\\\\", r"\\")
+                .strip()
+                for location in re.split(r"\s*(?<!(?<!\\)\\)[,;]\s*", locations)
+            ]
+            if x
         ]
 
 
@@ -340,7 +361,12 @@ class ProcWrapperParams(ConfigResolverParams):
         self.send_runtime_metadata: bool = True
 
         self.command: Optional[List[str]] = None
+        self.command_line: Optional[str] = None
+        self.shell_mode = SHELL_MODE_AUTO
+        self.strip_shell_wrapping = True
+        self.process_group_termination = True
         self.work_dir: str = "."
+
         self.process_timeout: Optional[int] = None
         self.process_max_retries: int = 0
         self.process_retry_delay: int = 0
@@ -356,6 +382,8 @@ class ProcWrapperParams(ConfigResolverParams):
         )
         self.status_update_interval: Optional[int] = None
 
+        self.include_timestamps_in_log: bool = True
+
         self.rollbar_access_token: Optional[str] = None
         self.rollbar_retries: Optional[int] = DEFAULT_ROLLBAR_RETRIES
         self.rollbar_retry_delay: int = DEFAULT_ROLLBAR_RETRY_DELAY_SECONDS
@@ -388,7 +416,7 @@ class ProcWrapperParams(ConfigResolverParams):
         )
 
         if not self.embedded_mode:
-            if not self.command:
+            if (not self.command) and (not self.command_line):
                 self._push_error(
                     process_errors,
                     "command",
@@ -463,6 +491,56 @@ class ProcWrapperParams(ConfigResolverParams):
     def run_mode_label(self) -> str:
         return "embedded" if self.embedded_mode else "wrapped"
 
+    def resolve_command_and_shell_flag(self) -> Tuple[Union[str, List[str]], bool]:
+        resolved_command: Union[str, List[str]] = (
+            self.command_line or self.command or ""
+        )
+
+        command_line = (
+            self.command_line if self.command_line else " ".join(self.command or "")
+        )
+
+        found_shell_wrapping = False
+
+        if self.strip_shell_wrapping:
+            done = False
+            while not done:
+                done = True
+                command_line_remainder, n = SHELL_WRAPPER_REGEX.subn("", command_line)
+
+                if n > 0:
+                    try:
+                        # Unquote
+                        command_remainder = shlex.split(command_line_remainder)
+
+                        # Only handle a single shell expression
+                        if len(command_remainder) == 1:
+                            # Switch to single string form
+                            resolved_command = command_remainder[0]
+
+                            _logger.info(
+                                f"Stripped shell wrapping from '{command_line}' to '{resolved_command}'"
+                            )
+
+                            command_line = resolved_command
+                            found_shell_wrapping = True
+                            done = False
+                    except Exception:
+                        _logger.exception(
+                            f"Error unquoting shell command '{command_line}'"
+                        )
+
+        shell_flag = True
+        if self.shell_mode == SHELL_MODE_FORCE_DISABLE:
+            shell_flag = False
+        elif (not found_shell_wrapping) and (self.shell_mode == SHELL_MODE_AUTO):
+            shell_flag = SHELL_COMMAND_REGEX.search(command_line) is not None
+
+        if (not shell_flag) and isinstance(resolved_command, str):
+            resolved_command = re.split(r"\s+", resolved_command)
+
+        return (resolved_command, shell_flag)
+
     def log_configuration(self) -> None:
         _logger.info(f"Run mode = {self.run_mode_label()}")
         _logger.info(f"Task Execution UUID = {self.task_execution_uuid}")
@@ -540,7 +618,9 @@ class ProcWrapperParams(ConfigResolverParams):
             _logger.debug("Rollbar is disabled")
 
         if not self.embedded_mode:
-            _logger.info(f"Command = {self.command}")
+            command, shell = self.resolve_command_and_shell_flag()
+            _logger.info(f"Command = {command}")
+            _logger.info(f"Use shell = {shell} (shell mode = {self.shell_mode})")
             _logger.info(f"Work dir = '{self.work_dir}'")
 
             enable_status_update_listener = self.enable_status_update_listener
@@ -637,6 +717,14 @@ class ProcWrapperParams(ConfigResolverParams):
     def _override_immutable_from_env(
         self, env: Dict[str, str], runtime_metadata: Optional[RuntimeMetadata]
     ) -> None:
+        self.include_timestamps_in_log = (
+            string_to_bool(
+                env.get("PROC_WRAPPER_INCLUDE_TIMESTAMPS_IN_LOG"),
+                default_value=self.include_timestamps_in_log,
+            )
+            or False
+        )
+
         self.offline_mode = (
             string_to_bool(
                 env.get("PROC_WRAPPER_OFFLINE_MODE"), default_value=self.offline_mode
@@ -868,7 +956,24 @@ class ProcWrapperParams(ConfigResolverParams):
                 ),
             )
 
-            self.command = self.command
+            self.command_line = env.get("PROC_WRAPPER_TASK_COMMAND", self.command_line)
+
+            self.shell_mode = env.get("PROC_WRAPPER_SHELL_MODE", self.shell_mode)
+
+            self.strip_shell_wrapping = (
+                string_to_bool(
+                    env.get("PROC_WRAPPER_STRIP_SHELL_WRAPPING"),
+                    default_value=self.strip_shell_wrapping,
+                )
+                or False
+            )
+
+            self.process_group_termination = coalesce(
+                string_to_bool(env.get("PROC_WRAPPER_TERMINATE_PROCESS_GROUP")),
+                self.process_group_termination,
+                True,
+            )
+
             self.work_dir = env.get("PROC_WRAPPER_WORK_DIR", self.work_dir)
 
         task_instance_metadata_str = env.get("PROC_WRAPPER_TASK_INSTANCE_METADATA")
@@ -1044,7 +1149,7 @@ def json_encoded(s: str):
     return json.loads(s)
 
 
-def make_arg_parser(require_command=True):
+def make_arg_parser():
     parser = argparse.ArgumentParser(
         prog="proc_wrapper",
         description="""
@@ -1055,8 +1160,7 @@ environment.
     """,
     )
 
-    if require_command:
-        parser.add_argument("command", nargs=argparse.REMAINDER)
+    parser.add_argument("command", nargs=argparse.REMAINDER)
 
     parser.add_argument(
         "-v", "--version", action="store_true", help="Print the version and exit"
@@ -1082,7 +1186,10 @@ UUID of Task (either the Task Name or the Task UUID must be specified)""",
     )
     task_group.add_argument(
         "--auto-create-task-run-environment-name",
-        help="Name of the Run Environment to use if auto-creating the Task (either the name or UUID of the Run Environment must be specified if auto-creating the Task). Defaults to the deployment name if the Run Environment UUID is not specified.",
+        help="""
+Name of the Run Environment to use if auto-creating the Task (either the name or
+UUID of the Run Environment must be specified if auto-creating the Task).
+Defaults to the deployment name if the Run Environment UUID is not specified.""",
     )
     task_group.add_argument(
         "--auto-create-task-run-environment-uuid",
@@ -1146,7 +1253,7 @@ for the schema.""",
     task_group.add_argument(
         "--max-concurrency",
         help="""
-Maximum number of concurrent Task Executions allowed with the same Task UUID.
+Maximum number of concurrent Task Executions of the same Task.
 Defaults to 1.""",
     )
     task_group.add_argument(
@@ -1209,7 +1316,7 @@ Defaults to {DEFAULT_API_RETRY_DELAY_SECONDS}.""",
         help=f"""
 Number of seconds to wait before resuming API requests, after retries are
 exhausted. Defaults to {DEFAULT_API_RESUME_DELAY_SECONDS}.
--1 means no resumption.""",
+-1 means to never resume.""",
     )
     api_group.add_argument(
         "--api-task-execution-creation-error-timeout",
@@ -1255,7 +1362,9 @@ Timeout for contacting API server, in seconds. Defaults to
         "-p",
         "--prevent-offline-execution",
         action="store_true",
-        help="Do not start processes if the API server is unavailable.",
+        help="""
+Do not start processes if the API server is unavailable or the wrapper is
+misconfigured.""",
     )
     api_group.add_argument(
         "-d", "--deployment", help="Deployment name (production, staging, etc.)"
@@ -1280,12 +1389,20 @@ Timeout for contacting API server, in seconds. Defaults to
         "-l",
         "--log-level",
         choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
-        default=os.environ.get("PROC_WRAPPER_LOG_LEVEL", _DEFAULT_LOG_LEVEL),
+        default=_DEFAULT_LOG_LEVEL,
         help="Log level",
     )
     log_group.add_argument(
         "--log-secrets", action="store_true", help="Log sensitive information"
     )
+    api_group.add_argument(
+        "--exclude-timestamps-in-log",
+        action="store_false",
+        dest="include_timestamps_in_log",
+        help="""
+Exclude timestamps in log (possibly because the log stream will be enriched by
+timestamps automatically by a logging service like AWS CloudWatch Logs)""",
+    )
 
     process_group = parser.add_argument_group("process", "Process settings")
     process_group.add_argument(
@@ -1294,11 +1411,61 @@ Timeout for contacting API server, in seconds. Defaults to
         default=".",
         help="Working directory. Defaults to the current directory.",
     )
+    process_group.add_argument(
+        "-c",
+        "--command-line",
+        default=".",
+        help="Command line to execute",
+    )
+    process_group.add_argument(
+        "--shell-mode",
+        choices=[
+            SHELL_MODE_AUTO,
+            SHELL_MODE_FORCE_ENABLE,
+            SHELL_MODE_FORCE_DISABLE,
+        ],
+        default=SHELL_MODE_AUTO,
+        help=f"""
+Indicates if the process command should be executed in a shell.
+Executing in a shell allows shell scripts, commands, and expressions to be used,
+with the disadvantage that termination signals may not be propagated to
+child processes.
+
+Options are:
+{SHELL_MODE_FORCE_ENABLE} -- Force the command to be executed in a shell;
+{SHELL_MODE_FORCE_DISABLE} -- Force the command to be executed without a shell;
+{SHELL_MODE_AUTO} -- Auto-detect the shell mode by analyzing the command.
+""",
+    )
+    process_group.add_argument(
+        "--no-strip-shell-wrapping",
+        action="store_false",
+        dest="strip_shell_wrapping",
+        help="""
+Do not strip the command-line of shell wrapping like "/bin/sh -c" that can be
+introduced by Docker when using shell form of ENTRYPOINT and CMD.
+""",
+    )
+    process_group.add_argument(
+        "--no-process-group-termination",
+        action="store_false",
+        dest="process_group_termination",
+        help="""
+Send termination and kill signals to the wrapped process only, instead of its
+process group (which is the default). Sending to the process group allows all
+child processes to receive the signals, even if the wrapped process does not
+forward signals. However, if your wrapped process manually handles and forwards
+signals to its child processes, you probably want to send signals to only
+your wrapped process.
+""",
+    )
+
     process_group.add_argument(
         "-t",
         "--process-timeout",
         help="""
-Timeout for process, in seconds. -1 means no timeout, which is the default.""",
+Timeout for process completion, in seconds. -1 means no timeout, which is the
+default.""",
     )
     process_group.add_argument(
         "-r",
@@ -1355,8 +1522,9 @@ The maximum number of bytes status update messages can be. Defaults to
     update_group.add_argument(
         "--status-update-interval",
         help="""
-Minimum of seconds to wait between sending status updates to the API server.
--1 means to not send status updates except with heartbeats. Defaults to -1.""",
+Minimum of number of seconds to wait between sending status updates to the API
+server. -1 means to not send status updates except with heartbeats. Defaults to
+-1.""",
     )
 
     config_group = parser.add_argument_group(
@@ -1377,7 +1545,6 @@ Minimum of seconds to wait between sending status updates to the API server.
     )
 
     config_group.add_argument(
-        "-c",
         "--config",
         action="append",
         dest="config_locations",
